"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ExpressionUtils = /** @class */ (function () {
    function ExpressionUtils(_defaultSeparator) {
        if (_defaultSeparator === void 0) { _defaultSeparator = "_"; }
        this._defaultSeparator = _defaultSeparator;
    }
    ExpressionUtils.prototype.getValueByExpression = function (instance, expression) {
        return expression(instance);
    };
    ExpressionUtils.prototype.getValueByProperties = function (instance, properties) {
        var _this = this;
        var result = instance;
        properties.forEach(function (property) {
            result = _this.getValue(result, property);
        });
        return result;
    };
    ExpressionUtils.prototype.getValue = function (instance, property) {
        if (property.indexOf(".") > -1) {
            return this.getValueByProperties(instance, property.split("."));
        }
        if (instance) {
            return instance[property];
        }
        return void 0;
    };
    ExpressionUtils.prototype.getColumnByExpression = function (expression, separator) {
        if (separator === void 0) { separator = this._defaultSeparator; }
        return this.getColumnByProperties(this.getPropertiesByExpression(expression), separator);
    };
    ExpressionUtils.prototype.getColumnByProperties = function (properties, separator) {
        if (separator === void 0) { separator = this._defaultSeparator; }
        return properties.join(separator);
    };
    ExpressionUtils.prototype.getPropertiesByExpression = function (expression) {
        var strAfterReturn = expression.toString().split("return")[1].trim();
        if (!strAfterReturn) {
            strAfterReturn = expression.toString().split("{")[1].trim();
        }
        var strBeforeSemicon = strAfterReturn.split(" ")[0].split(";")[0];
        return this.getPropertiesByExpressionString(strBeforeSemicon);
    };
    ExpressionUtils.prototype.getPropertiesByExpressionString = function (expression, alias) {
        if (alias === void 0) { alias = void 0; }
        if (this.hasExpression(expression, alias)) {
            var propertiesReferences = expression.split(".");
            if (propertiesReferences.length && this.hasExpression(expression, alias)) {
                propertiesReferences.shift();
            } // remove alias
            return propertiesReferences;
        }
        return [expression];
    };
    ExpressionUtils.prototype.getColumnByLambdaExpression = function (expression) {
        var propertiesMetadada = this.getPropertiesByLambdaExpression(expression);
        return {
            alias: propertiesMetadada.alias,
            columnLeft: this.getColumnByProperties(propertiesMetadada.propertiesLeft),
            columnRight: this.getColumnByProperties(propertiesMetadada.propertiesRight),
            operator: propertiesMetadada.operator,
        };
    };
    ExpressionUtils.prototype.getExpressionByLambdaExpression = function (expression) {
        var splitInitExpression = expression.toString().split("return");
        if (!splitInitExpression) {
            splitInitExpression = expression.toString().split("{");
        }
        var strAfterReturn = splitInitExpression[1].trim();
        var strExpression = strAfterReturn.split(";")[0].split(" ");
        if (strExpression.length !== 3) {
            throw new Error("Lambda expression '" + expression.toString() + "'\n            not supported! Use simple expression with '{expressionLeft} {operador} {expressionRight}'");
        }
        return {
            alias: this.getAlias(splitInitExpression[0]),
            expressionLeft: strExpression[0],
            expressionRight: strExpression[2],
            operator: strExpression[1],
        };
    };
    ExpressionUtils.prototype.getPropertiesByLambdaExpression = function (expression) {
        var expressionMetadada = this.getExpressionByLambdaExpression(expression);
        return {
            alias: expressionMetadada.alias,
            operator: expressionMetadada.operator,
            propertiesLeft: this.getPropertiesByExpressionString(expressionMetadada.expressionLeft, expressionMetadada.alias),
            propertiesRight: this.getPropertiesByExpressionString(expressionMetadada.expressionRight, expressionMetadada.alias),
        };
    };
    ExpressionUtils.prototype.getAlias = function (startExpression) {
        // https://stackoverflow.com/a/17779833/2290538
        var regexBetweenParentheses = /\(([^)]+)\)/;
        var resultRegex = regexBetweenParentheses.exec(startExpression);
        return resultRegex && resultRegex.length > 1 ? resultRegex[1] : "";
    };
    ExpressionUtils.prototype.hasAlias = function (value, alias) {
        if (!alias) {
            return true;
        }
        return value.indexOf(alias + ".") > -1;
    };
    ExpressionUtils.prototype.hasBetweenQuotes = function (value) {
        // let regexBetweenQuotes = /"[^"]*"/;
        var regexBetweenQuotes = /^(["'])(.*)\1$/;
        return regexBetweenQuotes.test(value);
    };
    ExpressionUtils.prototype.hasExpression = function (value, alias) {
        return !this.hasBetweenQuotes(value) && this.hasAlias(value, alias);
    };
    return ExpressionUtils;
}());
exports.ExpressionUtils = ExpressionUtils;
